
/**
 * Abstraktni trida pro elektricka zarizeni. Elektricka zarizeni se pripojuji do elektricke site
 * (zasunuti snury do zastrcky). Pokud jsou zarizeni zapnuta a zaroven jsou zapojena v siti,
 * odebiraji elektricky proud. Prilis velky odber ale muze zpusobit vypadek proudu v siti.
 * I pokud dojde k vypadku proudu, zustavaji zarizeni v puvodni stavu, tj. zapnuta zarizeni
 * zustavaji zapnuta a naopak.
 * <p/>
 * Elektricke zarizeni tedy muze zpusobit vypadek proudu ve dvou pripadech. (1) v okaziku, kdy se 
 * zapoji do site ve vypnutem stavu a pote se zapne. (2) pokud se zapojuje do site v zapnutem stavu.
 *
 * @author Ales Zlamal
 * @author Radek Oslejsek, oslejsek@fi.muni.cz
 */

public abstract class AbstractElectricDevice{

    private String name;
    private int voltage;
    private int wattage;
    
    /**
     * Constructor for objects of class AbstractElectricDevice
     * 
     * @param name nazev zarizeni
     * @param voltage napeti - fyzikalne: U [V = volty]
     * @param wattage vykon - fyzikalne: P [W = watty] : P = U * I, kde I [A = ampery]je proud
     * @throws IllegalArgumentException kdyz argument name je null nebo
     *                                  argumenty typu int jsou nekladne  
     */
    public AbstractElectricDevice(String name, int voltage, int wattage) {
        if (name == null) throw new IllegalArgumentException("sort");
        if (voltage <= 0) throw new IllegalArgumentException("voltage");
        if (wattage <= 0) throw new IllegalArgumentException("wattage");
        this.name = name;
        this.voltage = voltage;
        this.wattage = wattage;
    }

    String getName() {
        return name;
    }

    public int getWattage() {
        return wattage;
    }
    
    /**
     * Tato metoda vrati hodnotu odebiraneho proudu v amperech [A]
     * (presneji proudu, ktery by zarizeni odebiralo, kdyby spotrebovavalo jmenovity prikon)
     * 
     * @return hodnotu proudu, at zarizeni pracuje ci ne
     */
    public double getCurrent() {
        return ((double)wattage/(double)voltage);
    }

    @Override
    public String toString(){
        return name;
    }
    
    /*************************************************************************************
     *                                                                                   *
     *  abstraktni metody, ktere je treba  implementovat:                                *
     *                                                                                   *
     *************************************************************************************/

    /**
     * Zjisti, zda je zarizeni zapnuto (tj. jestli ma vypinac v poloze "zapnuto", bez ohledu na 
     * pripojeni k elektricke siti nebo stavu site).
     * 
     * @return true jestlize je zarizeni zapnuto, jinak false
     */
    abstract boolean isOn();
    
    /**
     * Vypne zarizeni, tj. prepne vypinac do stavu "vypnuto". Vypnutim zarizeni nikdy
     * nemuzeme zpusobit vypadek proudu.
     */
    abstract void switchOff();

    /**
     * Zapne zarizeni, tj. prepne vypinac do stavu "zapnuto". Jestlize je zarizeni pripojeno
     * k siti, vyvola metoda test jistice v elektricke siti. I v pripade, ze jistic vypne proud
     * z duvodu pretizeni, zustava zarizeni ve stavu "zapnuto".
     * <p/>
     * Tato metoda neselze atomicky. Nejprve se zmeni stav objektu na "zapnuto",
     * pak se teprve pripadne generuje vyjimka indikujici vypadek elektrickeho proudu.
     * 
     * @throws CircuitOverloadException pokud po zapnuti zarizeni doslo k vypadku proudu v siti
     */
    abstract void switchOn() throws CircuitOverloadException;
    
    /**
     * Pripojeni do site (zasunuti snury do zasuvky). Jestlize je zarizeni ve stavu "zapnuto",
     * muze zapojeni zpusobit vypadek proudu. Proto metoda v takovem pripade 
     * vyvola test jistice v elektricke siti.
     * 
     * @param  circuit elektricky obvod, do ktereho se zarizeni pripojuje
     * @throws java.lang.UnsupportedOperationException pokud je jiz zarizeni pripojeno 
     *         do jine site
     * @throws IllegalArgumentException pokud je argument circuit null
     * @throws CircuitOverloadException pokud doslo po zapojeni k vypadku proudu
     */
    abstract void plug(Circuit circuit) throws CircuitOverloadException;
    
    /**
     * Odpojeni od site (vytahnuti snury ze zasuvky). Tato operace nemuze nikdy zpusobit
     * vypadek site.
     * 
     * @throws java.lang.UnsupportedOperationException pokud neni toto zarizeni pripojeno
     *         do elektricke site
     */
    abstract void unplug() throws UnsupportedOperationException;

}
