
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
import java.util.TreeSet;

/**
 *
 * @author misko
 */
public class CircuitImpl implements Circuit {

    private final Set<AbstractElectricDevice> devices =
            new TreeSet<AbstractElectricDevice>();
    private boolean isOn = true;
    private final int maxCurrent;

    /**
     * Nove vytvorena elektricka sit bude pod proudem.
     *
     * @param maxCurrent Maximalni proud, ktery je mozno odebirat. Pokud zarizeni zapojena
     *                   do site odebiraji vice proudu nez je tato hodnota, jistic odpoji
     *                   elektrickou sit od proudu.
     *                   (fyzikalne: I [A = ampery] chrakteristka jistice)
     * @throws IllegalArgumentException je-li argument maxCurrent mensi nebo roven nule
     */
    public CircuitImpl(int maxCurrent) {
        this.maxCurrent = maxCurrent;
    }

    /**
     * Metoda simuluje cinnost jistice. Jestlize je sit bez proudu, metoda nedela nic.
     * Jestlize je pod proudem, zjisti aktualni zatizeni tak, ze secte proud odebirany 
     * vsemi zapojenymi ZAPNUTYMI zarizenimi (odber proudu elektrickym zarizenim viz metoda 
     * AbstractElectricDevice.getCurrent). V pripade pretizeni (soucet je vetsi jak 
     * maximalni proud) se proud vypne.
     * <p/>
     * Pozn: V metode nedojde k atomickemu selhani. Pri pretizeni site totiz nejprve 
     * metoda musi "vypnout proud" (tj. zmenit stav instance Circuit) a pak teprve 
     * vyhodit vyjimku.
     * 
     * @throws CircuitOverloadException pokud doslo k pretizeni a jistic vypnul proud
     */
    public void currentBreakerTest() throws CircuitOverloadException {
        if (isOn) {
            double current = 0;
            for (AbstractElectricDevice device : devices) {
                if (device.isOn()) {
                    current += device.getCurrent();
                }
            }
            if (current > maxCurrent) {
                isOn = false;
                throw new CircuitOverloadException("overload!!");
            }
        }
    }

    /**
     * Pokus o "nahozeni jistice", tj. pokus o opetovne zapnuti proudu v siti.
     * 
     * @throws CircuitOverloadException jestlize po zapnuti proudu jistic
     *         proud opet vypnul z duvodu pretizeni
     */
    public void restoreCurrent() throws CircuitOverloadException {
        isOn = true;
        currentBreakerTest();
    }

    /**
     * Pripojeni elektrickeho zarizeni do site (zapojeni do elektricke zasuvky).
     * Zarizeni muze zpusobit vypadek proudu pouze pokud je zapojovano do site
     * v zapnutem stavu.
     * 
     * @param device elektricke zarizeni, ktere je pripojovano do site
     * @throws IllegalArgumentException pokud je device null
     * @throws CircuitOverloadException pokud je zarizeni zapnute a jistic vypnul proud
     *         z duvodu pretizeni site
     */
    public void linkTo(ElectricDevice device) throws CircuitOverloadException {
        if (device == null) {
            throw new IllegalArgumentException("device is null");
        }
        devices.add(device);
        if (device.isOn()) {
            currentBreakerTest();
        }

    }

    private class DevicesComparator implements Comparator<AbstractElectricDevice> {

        public int compare(AbstractElectricDevice o1, AbstractElectricDevice o2) {
            return o1.getWattage() - o2.getWattage();
        }
    }

    void printDevices() {
        List<AbstractElectricDevice> list = new ArrayList<AbstractElectricDevice>();
        list.addAll(devices);
        Collections.sort(list, new DevicesComparator());
        for (AbstractElectricDevice abstractElectricDevice : list) {
            System.out.println(abstractElectricDevice);
        }
    }
    void printDevicesWithAnonymousClass() {
        List<AbstractElectricDevice> list = new ArrayList<AbstractElectricDevice>();
        list.addAll(devices);
        Collections.sort(list, new Comparator<AbstractElectricDevice>() {

            public int compare(AbstractElectricDevice o1, AbstractElectricDevice o2) {
                return o1.getWattage() - o2.getWattage();
            }
        });
        for (AbstractElectricDevice abstractElectricDevice : list) {
            System.out.println(abstractElectricDevice);
        }
    }

    /**
     * Odpoji elektricke zarizeni od site (vytazeni z elektricke zasuvky).
     * Pri odpojovani nemuze nikdy dojit k vypadku proudu.
     * 
     * @param device elektricke zarizeni, ktere je odpojovano ze site
     * @throws IllegalArgumentException pokud je device null, nebo pokud zarizeni
     *         neni zapojeno v teto siti
     */
    public void unlink(ElectricDevice device) {
        if (device == null) {
            throw new IllegalArgumentException("device is null");
        }

        if (!devices.remove(device)) {
            throw new IllegalArgumentException("device not in this circuit");
        }
    }
}
